#!/usr/bin/perl

use strict;
use warnings;
use Statistics::R;

my $zscoreweights = $ARGV[0];
my $clusters = $ARGV[1]; 
my $ncrnaname = lc($ARGV[2]); 


my @printarray = (); 
my @temparray = (); 
my @testpvalleastsqarray = ();
my $leasterror = 9999999;


open (MYDATA, $zscoreweights) or die ("Error: cannot open file $zscoreweights'\n");

my @zscoreweightlines = <MYDATA>;

close MYDATA;

my $orgcnt = scalar(@zscoreweightlines);
my $leastrho = 0;
my $fullpath = "/usr/bin/R";
my $R = Statistics::R->new( r_bin => $fullpath );



for (my $rho=0; $rho<=1; $rho=$rho + 0.1) { 
my @ltagarray = ();
my $ltag = '';
my $org = '';
my $pval = 0;
my $clnum = 1;
my $partlength = 0;
my %pvalorghash = (); 
my $clusternumber = 1;
                     

$R->startR;
my $Rinweights = $zscoreweights;
$R->set('myinfile', $Rinweights);
$R->set('orgcnt', $orgcnt);
my $therna = "$ncrnaname" . "_"; 
$R->set('therna', $therna);


$R->send(q`    

options(warn=-1)

weightdata <- read.table(myinfile, sep=";", header=FALSE)
weight <- ((as.numeric(weightdata$V2))**(1/(2.5))) # root 2.5
refids <- as.character(weightdata$V1)

##################################################
##                                              ##
## Retrieve Covariance Matrix from max clusters ##
##                                              ##
##################################################


data <- read.table("tags.clustered", sep=";", header=TRUE) # get data from hash clusters output

int <- data$clusternumber # get the clusternumbers as vect

# check this or all and save the ids if the length equals the organism count

# the vector with the saved ids can then be used to select the columns that match the ids.

# orgcnt muss man natuerlich mitgeben wenns nachher in der pipe lauuft
maxnums <- c()
for (i in 1:int[length(int)]) { # fuer jede clusternumber
    if(length(int[which(int==i)]) == orgcnt) {
        maxnums <- c(maxnums, i)
    }
}

# extract the lines which participate in maxclusters
maxclustdata <- data[data$clusternumber %in% maxnums, ]


# now work with the p-values in maxclustdata

#get the organism names (refseqs)
names <- unique(maxclustdata$sRNA_name)

#re-sort names vector to be the same as refids vector
namesresorted <- c()
for(i in 1:length(refids)) {
    check <- grep(refids[i], names, value=FALSE)
    namesresorted[i] <- as.character(names[check])
}


# do it for first as specific to initialize the dataframe
trueindices <- which(maxclustdata$sRNA_name==namesresorted[1])
truelines <- maxclustdata[trueindices, ]
thename <- sub(therna, "", namesresorted[1])
newframe <- data.frame(qnorm(truelines$p.value,lower.tail=FALSE))
names(newframe)[1]<-paste(thename)

for (i in 2:orgcnt) {
    # extract line indices from df for org i
    trueindices <- which(maxclustdata$sRNA_name==namesresorted[i])
    # put the actual lines in a vector
    truelines <- maxclustdata[trueindices, ]
    # merge the p-values into a data frame
    thename <- sub(therna, "", namesresorted[i])
    newframe <- data.frame(newframe, qnorm(truelines$p.value, lower.tail=FALSE))
    names(newframe)[i]<-paste(thename)
}

infindicespositive <- which(newframe==Inf) 
infindicesnegative <- which(newframe==-Inf)
infindices <- c()

if (! length(infindicespositive) == 0) {
for (i in 1:length(infindicespositive)) {
    if(infindicespositive[i] <= length(truelines$p.value)) {
        infindices <- c(infindices, infindicespositive[i])
    } else {
        deleterow <- infindicespositive[i]
        while (deleterow > length(truelines$p.value)) {
            deleterow <- deleterow - length(truelines$p.value)
        }
        infindices <- c(infindices, deleterow)
    }   
}
}

if (! length(infindicesnegative) == 0) {
for (i in 1:length(infindicesnegative)) {
    if(infindicesnegative[i] <= length(truelines$p.value)) {
        infindices <- c(infindices, infindicesnegative[i])
    } else {
        deleterow <- infindicesnegative[i]
        while (deleterow > length(truelines$p.value)) {
            deleterow <- deleterow - length(truelines$p.value)
        }
        infindices <- c(infindices, deleterow)
    }
}
}

if (length(infindices) > 0) {
    newframe <- newframe[-(infindices),]
}


covmatrix <- cov(newframe)
means <- c()

for (i in 1:length(newframe)) {
    means <- c(means, mean(newframe[i]))
}

condNormal <- function(x.given, mu, sigma, given.ind, req.ind){
# Returns conditional mean and variance of x[req.ind]
# Given x[given.ind] = x.given
# where X is multivariate Normal with
# mean = mu and covariance = sigma
#
B <- sigma[req.ind, req.ind]
C <- sigma[req.ind, given.ind, drop=FALSE]
D <- sigma[given.ind, given.ind]
CDinv <- C %*% solve(D)
cMu <- c(mu[req.ind] + CDinv %*% (x.given - mu[given.ind]))
cVar <- B - CDinv %*% t(C)
list(condMean=cMu, condVar=cVar)
}


`);


my $refidforR = "";
my $pvalsforR = "";
open(MYDATA, $clusters) or die("Error: cannot open file $clusters'\n");
$R->set('rho', $rho);
while ( my $line = <MYDATA> ) {
    if ($line =~ m/\d/) {               
        my @split = split(/;/, $line);
        $pval = $split[7];                         
        $clnum = $split[8];
        $ltag = $split[0];
        chomp $ltag;
        chomp $clnum;
        chomp $pval;
        if ( $split[3] =~ m/$ncrnaname\_(\S+)/ ) {
            chomp $1;
            $org = $1;
        }

    if ($clnum eq $clusternumber) {              
        $pvalorghash{$org} = $pval;
        push(@ltagarray, $ltag);
    }
    if ($clnum ne $clusternumber) {             
        for my $refid (keys %pvalorghash) {
            $refidforR = $refidforR . " " . $refid;
            $pvalsforR = $pvalsforR . " " . $pvalorghash{$refid};
        }
        
        $refidforR =~ s/^\s+//; #remove leading spaces
        $refidforR =~ s/\s+$//; #remove trailing spaces
        $pvalsforR =~ s/^\s+//; #remove leading spaces
        $pvalsforR =~ s/\s+$//; #remove trailing spaces
        $R->set('refidsincluster', $refidforR);
        $R->set('pvalsincluster', $pvalsforR);
	
        $refidforR = "";
        $pvalsforR = "";
        $R->send(q`
            pvalsincluster <- as.character(pvalsincluster)
            refidsincluster <- as.character(refidsincluster)
            theSplitPvals <- as.numeric(unlist(strsplit(pvalsincluster, split=" ")))
            theSplitRefids <- unlist(strsplit(refidsincluster, split=" "))
            p <- c()                    
   
            # hier wird korrekt sortiert nach der sortierung in zscore.weight
 
            for(i in 1:length(refids)) {
                for(j in 1:length(theSplitRefids)) {
                    if(refids[i] == theSplitRefids[j]) {
                        p[i] <- theSplitPvals[j]
                    }
                }
            }
            #print(p)
        `);

        $R->send(q`
            # hier muss ich das letzte element auf NA setzten falls es dieses noch nicht gibt
            # ansonsten wird nicht richtig aufgefüllt
            if(length(p) != length(refids)) {
                p[length(refids)] <- NA
            }
            indicesavailable <- c()
            indicesnotava <- c()
            for (i in 1:length(p)) {
                if(is.na(p[i])) {
                    indicesnotava <- c(indicesnotava, i)
                } else {
                    indicesavailable <- c(indicesavailable, i)
                }
            }             

            # befüllen der leeren slots mit den Werten aus condNormal
            p2 <- na.omit(p)
            p2 <- qnorm(p2, lower.tail = FALSE)
            cNresult <- condNormal(p2, means, covmatrix, indicesavailable, indicesnotava)
            p <- qnorm(p, lower.tail = FALSE)           
 
            cNcounter <- 1

            for (i in 1:length(p)) { 
                if(is.na(p[i])) { 
                    p[i] <- rnorm(1, mean=cNresult$condMean[cNcounter], sd=sqrt(cNresult$condVar[cNcounter,cNcounter])) 
                    cNcounter <- cNcounter + 1;
                } 
            }


            # formula 2.2 from "A Note on Combning Dependent Tests of Significance" Joachim Hartung
            cp <- pnorm(sum(weight * p)/sqrt((1-rho)*sum(weight^2)+rho*(sum(weight)^2)), lower.tail = FALSE)
            if(is.nan(cp) == TRUE) { cp <- 0 }
            print(cp)            
        `);

            my $ret = $R->read;
            $ret = substr $ret, 4;
            chomp $ret;
            push(@testpvalleastsqarray, $ret);
            %pvalorghash = ();
            my $arraypusher = $ret;
            foreach(@ltagarray) {
                    $arraypusher =  $arraypusher . ";" . $_;
            }
            push(@temparray, $arraypusher);

            $arraypusher = "";

            @ltagarray = ();
            push(@ltagarray, $ltag);
            $pvalorghash{$org} = $pval;
            $clusternumber++;
    }
    }
}


        for my $refid (keys %pvalorghash) {
            $refidforR = $refidforR . " " . $refid;
            $pvalsforR = $pvalsforR . " " . $pvalorghash{$refid};
        }

        $refidforR =~ s/^\s+//; #remove leading spaces
        $refidforR =~ s/\s+$//; #remove trailing spaces
        $pvalsforR =~ s/^\s+//; #remove leading spaces
        $pvalsforR =~ s/\s+$//; #remove trailing spaces
        $R->set('refidsincluster', $refidforR);
        $R->set('pvalsincluster', $pvalsforR);
        $refidforR = "";
        $pvalsforR = "";
        $R->send(q`
            pvalsincluster <- as.character(pvalsincluster)            
            refidsincluster <- as.character(refidsincluster)

            theSplitPvals <- as.numeric(unlist(strsplit(pvalsincluster, split=" ")))
            theSplitRefids <- unlist(strsplit(refidsincluster, split=" "))
            p <- c()

            # hier wird korrekt sortiert nach der sortierung in zscore.weight

            for(i in 1:length(refids)) {
                for(j in 1:length(theSplitRefids)) {
                    if(refids[i] == theSplitRefids[j]) {
                        p[i] <- theSplitPvals[j]
                    }
                }
            }
            #print(p)
        `);
 
        $R->send(q`
            # hier muss ich das letzte element auf NA setzten falls es dieses noch nicht gibt
            # ansonsten wird nicht richtig aufgefüllt
            if(length(p) != length(refids)) {
                p[length(refids)] <- NA
            }
            indicesavailable <- c()
            indicesnotava <- c()
            for (i in 1:length(p)) {
                if(is.na(p[i])) {
                    indicesnotava <- c(indicesnotava, i)
                } else {
                    indicesavailable <- c(indicesavailable, i)
                }
            }

            # befüllen der leeren slots mit den Werten aus condNormal
            p2 <- na.omit(p)
            p2 <- qnorm(p2, lower.tail = FALSE)
            cNresult <- condNormal(p2, means, covmatrix, indicesavailable, indicesnotava)
            p <- qnorm(p, lower.tail = FALSE)

            cNcounter <- 1

            for (i in 1:length(p)) {
                if(is.na(p[i])) {
                    p[i] <- rnorm(1, mean=cNresult$condMean[cNcounter], sd=sqrt(cNresult$condVar[cNcounter,cNcounter]))
                    cNcounter <- cNcounter + 1;
                }
            }


            # formula 2.2 from "A Note on Combning Dependent Tests of Significance" Joachim Hartung
            cp <- pnorm(sum(weight * p)/sqrt((1-rho)*sum(weight^2)+rho*(sum(weight)^2)), lower.tail = FALSE)
            if(is.nan(cp) == TRUE) { cp <- 0 }
            print(cp)
        `);

            my $ret = $R->read;
            $ret = substr $ret, 4;
            chomp $ret;
            push(@testpvalleastsqarray, $ret);
            %pvalorghash = ();
            my $arraypusher = $ret;
            foreach(@ltagarray) {
                    $arraypusher =  $arraypusher . ";" . $_;
            }
            push(@temparray, $arraypusher);

            $arraypusher = "";

            @ltagarray = ();
            push(@ltagarray, $ltag);
            $pvalorghash{$org} = $pval;
            $clusternumber++;



close MYDATA;

if (exists $printarray[0]) {
    $R->send(q`

    SplitPvalues <- c()

    `);
    my $newleasterror = 0;    
    foreach(@testpvalleastsqarray) {
        $R->set('thepvalue', $_);
        $R->send(q`
            if ((! is.nan(thepvalue)) && (is.numeric(thepvalue))) {
                SplitPvalues <- c(SplitPvalues, thepvalue)
            } ## else {
                ## print(thepvalue)
            ## }
            # printe mal die SplitPvalues laenge
        `);

    }   
    

    $R->send(q`
         histPvals <- hist(SplitPvalues, plot=FALSE)
    `);

    $R->send(q`
         error <- 0
         for(i in 1:length(histPvals$intensities)) {
             temperr <- ((1 - histPvals$intensities[i])**2)
             error <- error + temperr
         }
         print(error)

    `);
    $newleasterror = $R->read;
    $newleasterror = substr $newleasterror, 4;
    chomp $newleasterror;
    if($newleasterror < $leasterror) {
        $leastrho = $rho;
        $leasterror = $newleasterror;
        @printarray = @temparray;
    }

} else {
@printarray = @temparray;
    $R->send(q`

    SplitPvalues <- c()

    `);
    foreach(@testpvalleastsqarray) {
        $R->set('thepvalue', $_);
        $R->send(q`
            if ((! is.nan(thepvalue)) && (is.numeric(thepvalue))) {
                SplitPvalues <- c(SplitPvalues, thepvalue)
            } ##else {
                ##print(thepvalue)
            ##}
        `);
    }

    $R->send(q`
         histPvals <- hist(SplitPvalues, plot=FALSE)
    `);

    $R->send(q`
         error <- 0
         for(i in 1:length(histPvals$intensities)) {
             temperr <- ((1 - histPvals$intensities[i])**2)
             error <- error + temperr
         }
         print(error)

    `);
    $leasterror = $R->read;
    $leasterror = substr $leasterror, 4;
    $leastrho = $rho;
}


$R->stopR();
@testpvalleastsqarray = ();
@temparray = ();
}

open (MYFILE, '>>rhodevelopment.txt');
    print MYFILE "prelim. rho: $leastrho\n";
close (MYFILE);


for (my $rho=($leastrho - 0.1); $rho<=($leastrho + 0.1); $rho=$rho + 0.01) {          
if($rho<0) { last; }

my @ltagarray = ();
my $ltag = '';
my $org = '';
my $pval = 0;
my $clnum = 1;
my $partlength = 0;
my %pvalorghash = ();
my $clusternumber = 1;
$R->startR;
my $Rinweights = $zscoreweights;
$R->set('myinfile', $Rinweights);
$R->set('orgcnt', $orgcnt);
my $therna = "$ncrnaname" . "_";
$R->set('therna', $therna);


$R->send(q`    

options(warn=-1)

weightdata <- read.table(myinfile, sep=";", header=FALSE)
weight <- ((as.numeric(weightdata$V2))**(1/(2.5))) # root 2.5
refids <- as.character(weightdata$V1)

##################################################
##                                              ##
## Retrieve Covariance Matrix from max clusters ##
##                                              ##
##################################################


data <- read.table("tags.clustered", sep=";", header=TRUE) # get data from hash clusters output

int <- data$clusternumber # get the clusternumbers as vect

# check this or all and save the ids if the length equals the organism count

# the vector with the saved ids can then be used to select the columns that match the ids.

# orgcnt muss man natuerlich mitgeben wenns nachher in der pipe lauuft
maxnums <- c()
for (i in 1:int[length(int)]) { # fuer jede clusternumber
    if(length(int[which(int==i)]) == orgcnt) {
        maxnums <- c(maxnums, i)
    }
}

# extract the lines which participate in maxclusters
maxclustdata <- data[data$clusternumber %in% maxnums, ]


# now work with the p-values in maxclustdata

#get the organism names (refseqs)
names <- unique(maxclustdata$sRNA_name)

#re-sort names vector to be the same as refids vector
namesresorted <- c()
for(i in 1:length(refids)) {
    check <- grep(refids[i], names, value=FALSE)
    namesresorted[i] <- as.character(names[check])
}


# do it for first as specific to initialize the dataframe
trueindices <- which(maxclustdata$sRNA_name==namesresorted[1])
truelines <- maxclustdata[trueindices, ]
thename <- sub(therna, "", namesresorted[1])
newframe <- data.frame(qnorm(truelines$p.value,lower.tail=FALSE))
names(newframe)[1]<-paste(thename)

for (i in 2:orgcnt) {
    # extract line indices from df for org i
    trueindices <- which(maxclustdata$sRNA_name==namesresorted[i])
    # put the actual lines in a vector
    truelines <- maxclustdata[trueindices, ]
    # merge the p-values into a data frame
    thename <- sub(therna, "", namesresorted[i])
    newframe <- data.frame(newframe, qnorm(truelines$p.value, lower.tail=FALSE))
    names(newframe)[i]<-paste(thename)
}

infindicespositive <- which(newframe==Inf) 
infindicesnegative <- which(newframe==-Inf)
infindices <- c()

if (! length(infindicespositive) == 0) {
for (i in 1:length(infindicespositive)) {
    if(infindicespositive[i] <= length(truelines$p.value)) {
        infindices <- c(infindices, infindicespositive[i])
    } else {
        deleterow <- infindicespositive[i]
        while (deleterow > length(truelines$p.value)) {
            deleterow <- deleterow - length(truelines$p.value)
        }
        infindices <- c(infindices, deleterow)
    }   
}
}

if (! length(infindicesnegative) == 0) {
for (i in 1:length(infindicesnegative)) {
    if(infindicesnegative[i] <= length(truelines$p.value)) {
        infindices <- c(infindices, infindicesnegative[i])
    } else {
        deleterow <- infindicesnegative[i]
        while (deleterow > length(truelines$p.value)) {
            deleterow <- deleterow - length(truelines$p.value)
        }
        infindices <- c(infindices, deleterow)
    }
}
}

if (length(infindices) > 0) {
    newframe <- newframe[-(infindices),]
}


covmatrix <- cov(newframe)
means <- c()

for (i in 1:length(newframe)) {
    means <- c(means, mean(newframe[i]))
}

condNormal <- function(x.given, mu, sigma, given.ind, req.ind){
# Returns conditional mean and variance of x[req.ind]
# Given x[given.ind] = x.given
# where X is multivariate Normal with
# mean = mu and covariance = sigma
#
B <- sigma[req.ind, req.ind]
C <- sigma[req.ind, given.ind, drop=FALSE]
D <- sigma[given.ind, given.ind]
CDinv <- C %*% solve(D)
cMu <- c(mu[req.ind] + CDinv %*% (x.given - mu[given.ind]))
cVar <- B - CDinv %*% t(C)
list(condMean=cMu, condVar=cVar)
}


`);


my $refidforR = "";
my $pvalsforR = "";
open(MYDATA, $clusters) or die("Error: cannot open file $clusters'\n");
$R->set('rho', $rho);
while ( my $line = <MYDATA> ) {
    if ($line =~ m/\d/) {              
        my @split = split(/;/, $line);
        $pval = $split[7];                         
        $clnum = $split[8];
        $ltag = $split[0];
        chomp $ltag;
        chomp $clnum;
        chomp $pval;
        if ( $split[3] =~ m/$ncrnaname\_(\S+)/ ) {
            chomp $1;
            $org = $1;
        }

    if ($clnum eq $clusternumber) {              
        $pvalorghash{$org} = $pval;
        push(@ltagarray, $ltag);
    }
    if ($clnum ne $clusternumber) {             
        for my $refid (keys %pvalorghash) {
            $refidforR = $refidforR . " " . $refid;
            $pvalsforR = $pvalsforR . " " . $pvalorghash{$refid};
        }
        
        $refidforR =~ s/^\s+//; #remove leading spaces
        $refidforR =~ s/\s+$//; #remove trailing spaces
        $pvalsforR =~ s/^\s+//; #remove leading spaces
        $pvalsforR =~ s/\s+$//; #remove trailing spaces
        $R->set('refidsincluster', $refidforR);
        $R->set('pvalsincluster', $pvalsforR);
	
        $refidforR = "";
        $pvalsforR = "";
        $R->send(q`
            pvalsincluster <- as.character(pvalsincluster)
            refidsincluster <- as.character(refidsincluster)
            theSplitPvals <- as.numeric(unlist(strsplit(pvalsincluster, split=" ")))
            theSplitRefids <- unlist(strsplit(refidsincluster, split=" "))
            p <- c()                    
   
            # hier wird korrekt sortiert nach der sortierung in zscore.weight
 
            for(i in 1:length(refids)) {
                for(j in 1:length(theSplitRefids)) {
                    if(refids[i] == theSplitRefids[j]) {
                        p[i] <- theSplitPvals[j]
                    }
                }
            }
            #print(p)
        `);

        $R->send(q`
            # hier muss ich das letzte element auf NA setzten falls es dieses noch nicht gibt
            # ansonsten wird nicht richtig aufgefüllt
            if(length(p) != length(refids)) {
                p[length(refids)] <- NA
            }
            indicesavailable <- c()
            indicesnotava <- c()
            for (i in 1:length(p)) {
                if(is.na(p[i])) {
                    indicesnotava <- c(indicesnotava, i)
                } else {
                    indicesavailable <- c(indicesavailable, i)
                }
            }             

            # befüllen der leeren slots mit den Werten aus condNormal
            p2 <- na.omit(p)
            p2 <- qnorm(p2, lower.tail = FALSE)
            cNresult <- condNormal(p2, means, covmatrix, indicesavailable, indicesnotava)
            p <- qnorm(p, lower.tail = FALSE)           
 
            cNcounter <- 1

            for (i in 1:length(p)) { 
                if(is.na(p[i])) { 
                    p[i] <- rnorm(1, mean=cNresult$condMean[cNcounter], sd=sqrt(cNresult$condVar[cNcounter,cNcounter])) 
                    cNcounter <- cNcounter + 1;
                } 
            }


            # formula 2.2 from "A Note on Combning Dependent Tests of Significance" Joachim Hartung
            cp <- pnorm(sum(weight * p)/sqrt((1-rho)*sum(weight^2)+rho*(sum(weight)^2)), lower.tail = FALSE)
            if(is.nan(cp) == TRUE) { cp <- 0 } ########################################################################### workaround for the NaN problem
            print(cp)            
        `);

            my $ret = $R->read;
            $ret = substr $ret, 4;
            chomp $ret;
            push(@testpvalleastsqarray, $ret);
            %pvalorghash = ();
            my $arraypusher = $ret;
            foreach(@ltagarray) {
                    $arraypusher =  $arraypusher . ";" . $_;
            }
            push(@temparray, $arraypusher);

            $arraypusher = "";

            @ltagarray = ();
            push(@ltagarray, $ltag);
            $pvalorghash{$org} = $pval;
            $clusternumber++;
    }
    }
}


        for my $refid (keys %pvalorghash) {
            $refidforR = $refidforR . " " . $refid;
            $pvalsforR = $pvalsforR . " " . $pvalorghash{$refid};
        }

        $refidforR =~ s/^\s+//; #remove leading spaces
        $refidforR =~ s/\s+$//; #remove trailing spaces
        $pvalsforR =~ s/^\s+//; #remove leading spaces
        $pvalsforR =~ s/\s+$//; #remove trailing spaces
        $R->set('refidsincluster', $refidforR);
        $R->set('pvalsincluster', $pvalsforR);
        $refidforR = "";
        $pvalsforR = "";
        $R->send(q`
            pvalsincluster <- as.character(pvalsincluster)            
            refidsincluster <- as.character(refidsincluster)

            theSplitPvals <- as.numeric(unlist(strsplit(pvalsincluster, split=" ")))
            theSplitRefids <- unlist(strsplit(refidsincluster, split=" "))
			    p <- c()

		    # hier wird korrekt sortiert nach der sortierung in zscore.weight

		    for(i in 1:length(refids)) {
			for(j in 1:length(theSplitRefids)) {
			    if(refids[i] == theSplitRefids[j]) {
				p[i] <- theSplitPvals[j]
			    }
			}
		    }
		    #print(p)
		`);
	 
		$R->send(q`
		    # hier muss ich das letzte element auf NA setzten falls es dieses noch nicht gibt
		    # ansonsten wird nicht richtig aufgefüllt
		    if(length(p) != length(refids)) {
			p[length(refids)] <- NA
		    }
		    indicesavailable <- c()
		    indicesnotava <- c()
		    for (i in 1:length(p)) {
			if(is.na(p[i])) {
			    indicesnotava <- c(indicesnotava, i)
			} else {
			    indicesavailable <- c(indicesavailable, i)
			}
		    }

		    # befüllen der leeren slots mit den Werten aus condNormal
		    p2 <- na.omit(p)
		    p2 <- qnorm(p2, lower.tail = FALSE)
		    cNresult <- condNormal(p2, means, covmatrix, indicesavailable, indicesnotava)
		    p <- qnorm(p, lower.tail = FALSE)

		    cNcounter <- 1

		    for (i in 1:length(p)) {
			if(is.na(p[i])) {
			    p[i] <- rnorm(1, mean=cNresult$condMean[cNcounter], sd=sqrt(cNresult$condVar[cNcounter,cNcounter]))
			    cNcounter <- cNcounter + 1;
			}
		    }


		    # formula 2.2 from "A Note on Combning Dependent Tests of Significance" Joachim Hartung
		    cp <- pnorm(sum(weight * p)/sqrt((1-rho)*sum(weight^2)+rho*(sum(weight)^2)), lower.tail = FALSE)
		    if(is.nan(cp) == TRUE) { cp <- 0 }
                    print(cp)
		`);

		    my $ret = $R->read;
		    $ret = substr $ret, 4;
		    chomp $ret;
		    push(@testpvalleastsqarray, $ret);
		    %pvalorghash = ();
		    my $arraypusher = $ret;
		    foreach(@ltagarray) {
			    $arraypusher =  $arraypusher . ";" . $_;
		    }
		    push(@temparray, $arraypusher);

		    $arraypusher = "";

		    @ltagarray = ();
		    push(@ltagarray, $ltag);
		    $pvalorghash{$org} = $pval;
		    $clusternumber++;


	close MYDATA;



	if (exists $printarray[0]) {
	    $R->send(q`

	    SplitPvalues <- c()

	    `);
	    my $newleasterror = 0;    
	    foreach(@testpvalleastsqarray) {
		$R->set('thepvalue', $_);
		$R->send(q`
		    if ((! is.nan(thepvalue)) && (is.numeric(thepvalue))) {
			SplitPvalues <- c(SplitPvalues, thepvalue)
		    } ## else {
			## print(thepvalue)
		    ## }
		    # printe mal die SplitPvalues laenge
		`);
	    }   

	    $R->send(q`
		 histPvals <- hist(SplitPvalues, plot=FALSE)
	    `);
	    $R->send(q`
		 error <- 0
		 for(i in 1:length(histPvals$intensities)) {
		     temperr <- ((1 - histPvals$intensities[i])**2)
		     error <- error + temperr
		 }
		 print(error)

	    `);
	    $newleasterror = $R->read;
	    $newleasterror = substr $newleasterror, 4;
	    chomp $newleasterror;
	    if($newleasterror < $leasterror) {
		open (MYFILE, '>>rhodevelopment.txt');
		    print MYFILE "final rho: $rho\n";
		close (MYFILE);
		$leasterror = $newleasterror;
		@printarray = @temparray;
	    }

	} else {
	@printarray = @temparray;
	    $R->send(q`

	    SplitPvalues <- c()

	    `);
	    foreach(@testpvalleastsqarray) {
		$R->set('thepvalue', $_);
		$R->send(q`
		    if ((! is.nan(thepvalue)) && (is.numeric(thepvalue))) {
			SplitPvalues <- c(SplitPvalues, thepvalue)
		    } ## else {
		       ## print(thepvalue)
		    ## }
		`);
	    }


	    $R->send(q`
         histPvals <- hist(SplitPvalues, plot=FALSE)
    `);

    $R->send(q`
         error <- 0
         for(i in 1:length(histPvals$intensities)) {
             temperr <- ((1 - histPvals$intensities[i])**2)
             error <- error + temperr
         }
         print(error)

    `);
    $leasterror = $R->read;
    $leasterror = substr $leasterror, 4;
}


$R->stopR();
@testpvalleastsqarray = ();
@temparray = ();
}

foreach(@printarray) {print "$_;\n";}


